@@ -1,10 +1,11 @@
 from typing import List
 
+import numpy as np
+from duckietown_world import pose_from_friendly
+from duckietown_world.utils import SE2_apply_R2
 from geometry import SE2, SE2value
 from zuper_commons.logs import ZLogger
 
-from duckietown_world import pose_from_friendly
-from duckietown_world.utils import SE2_apply_R2
 from .collision_protocol import Circle, PlacedPrimitive, Rectangle
 
 logger = ZLogger(__name__)
@@ -13,19 +14,24 @@ __all__ = ["plot_geometry"]
 
 
 # noinspection PyListCreation
-def plot_geometry(ax, se0: SE2value, structure: List[PlacedPrimitive], style, zorder: int, text: str = None ):
+def plot_geometry(ax, se0: SE2value, structure: List[PlacedPrimitive], style, zorder: int,
+                  text: str = None) -> List:
     from matplotlib import pyplot
-
+    artists = []
     ax: pyplot.Axes
     for pp in structure:
         if text is not None:
             x, y = pp.pose.x, pp.pose.y
             x, y = SE2_apply_R2(se0, (x, y))
-            ax.add_artist(pyplot.Text(text=text, x=x, y=y, zorder=zorder + 1))
+            a = pyplot.Text(text=text, x=x, y=y, zorder=zorder + 1)
+            artists.append(a)
+            ax.add_artist(a)
         if isinstance(pp.primitive, Circle):
             x, y = pp.pose.x, pp.pose.y
             x, y = SE2_apply_R2(se0, (x, y))
-            ax.add_artist(pyplot.Circle((x, y), pp.primitive.radius, fill=True, color=style, zorder=zorder))
+            a = pyplot.Circle((x, y), pp.primitive.radius, fill=True, color=style, zorder=zorder)
+            ax.add_artist(a)
+            artists.append(a)
         if isinstance(pp.primitive, Rectangle):
             q = SE2.multiply(se0, pose_from_friendly(pp.pose))
             points = []
@@ -38,7 +44,13 @@ def plot_geometry(ax, se0: SE2value, structure: List[PlacedPrimitive], style, zo
             xs = [_[0] for _ in points]
             ys = [_[1] for _ in points]
 
-            pyplot.fill(xs, ys, "-", color=style, zorder=zorder)
+            xy = np.array((xs, ys)).T
+            # print(xy)
+            a = pyplot.Polygon(xy, fill=True, edgecolor=None, facecolor=style, zorder=zorder)
+            # ax            a = pyplot.fill(xs, ys, "-", color=style, zorder=zorder)
+            ax.add_artist(a)
+            artists.append(a)
         # if isinstance(pp.primitive, Rectanlge):
         #     x, y = pp.pose.x, pp.pose.y
         #     ax.add_artist(pyplot.Circle((x, y), pp.primitive.radius, fill=False, color=style))
+    return artists
